import time
import argparse
import statistics
import numpy as np
from rouge import Rouge
from sklearn.metrics import f1_score

def eval(args):
    with open('./task1_evidence_detection/test.txt', 'r', encoding='UTF-8') as f:
        label_true = [line.strip().split('\t')[-1] for line in f.readlines()]
    with open(args.task1_file_path, 'r', encoding='UTF-8') as f:
        label_pred = [line.strip() for line in f.readlines()]
    score1 = f1_score(label_true, label_pred, pos_label='1')

    with open('./task2_stance_classification/test.txt', 'r', encoding='UTF-8') as f:
        label_true = [line.strip().split('\t')[-1] for line in f.readlines()]
    with open(args.task2_file_path, 'r', encoding='UTF-8') as f:
        label_pred = [line.strip() for line in f.readlines()]
    score2 = f1_score(label_true, label_pred, pos_label='1')

    rouge = Rouge()
    with open('./task3_counter_argument_generation/test.txt', 'r', encoding='UTF-8') as f:
        refs = [line.strip().split('\t')[-1] for line in f.readlines()]
    with open(args.task3_file_path, 'r', encoding='UTF-8') as f:
        hyps = [line.strip() for line in f.readlines()]
    scores = [rouge.get_scores(hyp, ref)[0]['rouge-l']['f'] for ref, hyp in zip(refs, hyps)]
    score3 = statistics.mean(scores)

    print('-'*10, '测试结果', '-'*10)
    print(f'队伍：{args.team_name}')
    print(f'时间：{time.strftime("%Y-%m-%d %H:%M:%S")}')
    print('任务一：{:.2f}'.format(100*score1))
    print('任务二：{:.2f}'.format(100*score2))
    print('任务三：{:.2f}'.format(100*score3))



if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--team_name', type=str, default='xx队', help='队伍名')
    parser.add_argument('--task1_file_path', type=str, default='./task1_evidence_detection/output.txt', help='任务一输出文件路径')
    parser.add_argument('--task2_file_path', type=str, default='./task2_stance_classification/output.txt', help='任务二输出文件路径')
    parser.add_argument('--task3_file_path', type=str, default='./task3_counter_argument_generation/output.txt', help='任务三输出文件路径')
    args = parser.parse_args()
    eval(args)