import os
import time
import jieba
import argparse
import statistics
import numpy as np
from rouge import Rouge
from sklearn.metrics import f1_score

def eval(args):
    rouge = Rouge()
    test_folder = "./test"
    all_rouge_scores = []

    test_filenames = {f for f in os.listdir(test_folder) if f.endswith(".txt")}
    output_filenames = {f for f in os.listdir(args.output_folder_path) if f.endswith(".txt")}

    if test_filenames != output_filenames:
        raise ValueError(f"辩题匹配错误:\n输出文件夹中有缺漏辩题或与测试辩题不匹配，请仔细检查!")
    
    for filename in output_filenames:
        output_file_path = os.path.join(args.output_folder_path, filename)
        with open(output_file_path, 'r', encoding='utf-8') as file:
            if sum(1 for _ in file) != 5:
                raise ValueError(f"论点数量错误：请确保每个辩题下的输出文件内含有5个论点！\n建议检查'{args.output_folder_path}/{filename}'文件。")
    
    # 遍历测试文件夹中的所有txt文件
    for filename in os.listdir(test_folder):
        # 确保是文件且以.txt结尾
        if filename.endswith(".txt") and os.path.isfile(os.path.join(test_folder, filename)):
            # 打开测试文件夹中的文件和输出文件夹中的对应文件
            with open(os.path.join(test_folder, filename), 'r', encoding='utf-8') as test_file, \
                open(os.path.join(args.output_folder_path, filename), 'r', encoding='utf-8') as output_file:
                
                # 读取文件中的论点列表
                test_lines = test_file.readlines()
                output_lines = output_file.readlines()
                
                # 计算ROUGE-L分数
                max_rouge_score = 0
                for test_line in test_lines:
                    for output_line in output_lines:
                        # 计算单个论点的ROUGE-L分数
                        test_line = ' '.join(jieba.cut(test_line))
                        output_line = ' '.join(jieba.cut(output_line))
                        scores = rouge.get_scores(output_line, test_line)
                        rouge_l_score = scores[0]['rouge-l']['f']
                        max_rouge_score = max(max_rouge_score, rouge_l_score)
                
                # 将最大ROUGE-L分数添加到列表中
                all_rouge_scores.append(max_rouge_score)

    # 计算所有辩题的平均ROUGE-L分数
    score = sum(all_rouge_scores) / len(all_rouge_scores)
    print('-'*10, '阶段二-任务一 测试结果', '-'*10)
    print(f'队伍：{args.team_name}')
    print(f'时间：{time.strftime("%Y-%m-%d %H:%M:%S")}')
    print('任务得分：{:.2f}'.format(100*score))

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--team_name', type=str, default='xx队', help='队伍名')
    parser.add_argument('--output_folder_path', type=str, default='./output', help='输出文件夹路径')
    args = parser.parse_args()
    eval(args)
